#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

#############################################################################


from Products.CMFFolder.config import *

from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo

from Persistence import Persistent

#from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import getSecurityManager, SpecialUsers
from Acquisition import aq_base, aq_parent, Acquired, Implicit
import string


#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseFolderSchema +  Schema((
    StringField('title',
                searchable=1,
                required=1,
                ),
#    TextField('description',
#              searchable=1,
#              widget=VisualWidget,
#              ),
    StringField ('acquire_style',
              write_permission=CMFCorePermissions.ModifyPortalContent,
              vocabulary=YESNO,
              default='1',
              required=1,
              widget=SelectionWidget(label="Use acquired style",
                  description="Select 'No' only if you want to create your own " + \
                              "style sheet and so do the objects inside this folder. " + \
                              "Otherwise, the acquired style will be used."
                 ),
        ),
    TextField('css_fields',
              default='',
              visible=1,
              widget=TextAreaWidget(label="Cascading Style Sheet",
                         ),
              ),

    StringField('quota_bytes',
              required=1,
              default='0',
              widget=StringWidget(
                  label="Quota Size in Bytes",
                  description="(0 for no limit)",
                  ),
              ),
    StringField('quota_objects',
              required=1,
              default='0',
              widget=StringWidget(
                  label="Maximum number of Objects",
                  description="(0 for unlimited)",
                  ),
              ),
    StringField('quota_maxsize',
              required=1,
              default='0',
              widget=StringWidget(
                  label="Maximum Object Size in Bytes",
                  description="(0 for unlimited)",
                  ),
              ),
    StringField ('allow_nested',
              write_permission=CMFCorePermissions.ModifyPortalContent,
              vocabulary=YESNO,
              default='1',
              required=1,
              widget=SelectionWidget(label="Allow Nested",
                  description="Allow creating CMFFolder insider another CMFFolder"
                 ),
        ),
    ))


class CMFFolder(BaseFolder, Persistent, Implicit):
    """Post your notes, so everyone can know what you want, and what you provide
    """
    security = ClassSecurityInfo()
    
    schema = schema
    archetype_name = "CMF Folder"
    
    #allow_discussion = 1
    
    global_allow = 1
    
    
    filter_content_types = 0
    
    actions = (
        )
        
    #############################################################################
    ##                             Basics                                      ##
    #############################################################################

    __ac_permissions__= \
        BaseFolder.__ac_permissions__ +  \
        (
        ('View current quota', ('currentCount','currentSize')),
        )
        
    #############################################################################
    ##                             Utils                                       ##
    #############################################################################

    def _check_quota(self, filechange, sizechange=0, objsize=0):
        """ Check if the change in usage violates the quota settings """
        #
        # print "I'm a quota folder: ", filechange, sizechange, objsize
        # print "Current quota: ", self._quota_filecount, self._quota_size 

        #
        # The emergency user can create *some* objects (i.e. userfolders).
        # make sure this never gives a QuotaExceeded exception

        # get tools
        #catalog = self.portal_catalog
        #membership = self.portal_membership
        
        #items= self.portal_catalog(path=self.absolute_url('relative'))
        _quota_filecount = self.get_current_count()
        _quota_size = self.get_current_size() 
        quota_objects = self.get_quota_objects()
        quota_bytes = self.get_quota_bytes()
        quota_maxsize = self.get_quota_maxsize()
        emergency = 0
        user=getSecurityManager().getUser()
        
        if (SpecialUsers.emergency_user and
            aq_base(user) is SpecialUsers.emergency_user):
            LOG('QuotaFolder', INFO, 
                "Emergency user detected - not enforcing quota")
            emergency = 1
        
        if not emergency and \
           filechange > 0 and \
           _quota_filecount + filechange > quota_objects \
                                              > 0:
            raise QuotaExceededException, \
                  "Too many files, this operation would require a quotum " + \
                  "of %d files, your current quotum is %d files" % \
                  (_quota_filecount + filechange, quota_objects)
                  
        if not emergency and \
           sizechange > 0 and \
           _quota_size + sizechange > quota_bytes > 0:
            raise QuotaExceededException, \
                 "Too much space, this operation would require a " + \
                 "quotum of %d bytes, your current quotum is %d bytes" % \
                 (_quota_size + sizechange, quota_bytes)
                 
        if not emergency and objsize > quota_maxsize > 0:
            raise QuotaExceededException, \
                  "The object is too large. It requires " + \
                  "%s bytes, your maximum object size limit is %d bytes" % \
                  (objsize, quota_maxsize)

        #self._quota_filecount = self._quota_filecount + filechange
        #self._quota_size = self._quota_size + sizechange

        ##
        ## This should not happen... 
        path = string.join(self.getPhysicalPath(), "/")
        if _quota_filecount < 0:
            LOG('QuotaFolder', WARNING, "Negative quota filecount in %s" % path)
            #if not self._debug:
               # _quota_filecount = 0
        if _quota_size < 0:
            LOG('QuotaFolder', WARNING, "Negative quota size in %s" % path)
            #if not self._debug:
             #   self._quota_size = 0

        # print "New quota: ", self._quota_filecount, self._quota_size     

    def get_current_count(self):
        return len(self.portal_catalog(path=self.absolute_url('relative')  )) -1  #(minus self)

    def current_size(self):
        items= self.portal_catalog(path=self.absolute_url('relative') )
        sum = 0
    
        for i in items:
            
            a = i.getObject()
            sum = sum + int(i.getObjectSize(a))
        return sum

    def get_current_filecount(self):
        return self.current_count()

    def get_current_size(self):
        return self.current_size()


    def get_quota_bytes(self):
        return int(self.quota_bytes)

    def get_quota_objects(self):
        return int(self.quota_objects)

    def get_quota_maxsize(self):
        return int(self.quota_maxsize)

    def getObjectSize(indexedObj, object):
        """ 
            attempt to determine number of size and number of subobjects of object 
        """
        ##
        ## ZPT doesn't return the size of the unrendered content it seems, 
        ## which leads to
        ## very inconsistent sizes. Let's hope this fixes it...
        obj = aq_base(object)
        sum = 0
        sum = sum + len(indexedObj.SearchableText())
        
        if hasattr(obj, "_get_usage"):
            tmp= len(obj._get_usage())
            if tmp > 0:
                sum = sum + tmp
        elif hasattr(obj, "get_size"):
            ##
            ## Some products (such as PhotoFolders Photo) raise an exception
            ## when requested their size at init time
            try:
                tmp = obj.get_size()
                if tmp > 0:
                    sum = sum + tmp
            except: # can be any kind of exception
                LOG('CMFFolderProduct', BLATHER, 'Failed to get size for %s/%s' % \
                     (getid(obj.id), obj.meta_type))
        elif hasattr(obj, "getSize"):
            ## getSize is deprecated, but if it's all there is...
            tmp = obj.getSize()
            if tmp > 0:
                sum = sum + tmp
        else:
            LOG('CMFFolderProduct', BLATHER,  "Can't get size for %s/%s" % \
                (getid(obj.id), obj.meta_type))
        return sum


    security.declarePublic( 'getFileSize' )
    def getFileSize(self, bytes):
    
        try:
            
            """
            Format a number of bytes as bytes, K, or MB.
            """
            megs=bytes / 1048576.0
            if megs > 1:
                return "%.2f MB" % megs
            k=bytes / 1024.0
            if k > 1:
                return "%d K" % k
            return "%d bytes" % bytes
        except:
            return None

    def manage_afterAdd(self, item, container):
        # print "afterAdd %s %s"%(str(item), str(container))
        allow_nested= getattr(self, 'allow_nested', '0')
        allow_nested = int(allow_nested)
        if not allow_nested:
            for parent in self.aq_chain[1:]:
                if getattr(parent, "meta_type", '') == "CMFFolder":
                    raise CMFFolderNestedException, \
                          "Cannot add a CMFFolder inside another CMFFolder"

        BaseFolder.manage_afterAdd(self, item, container)
          


registerType(CMFFolder)
